package cn.com.connor.bh.bhdemo.demos.web.service;

import cn.com.connor.bh.bhdemo.demos.web.entity.MinioPojo;
import io.minio.*;
import io.minio.messages.Item;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.InvalidKeyException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Component
public class MinIOService {
    private static final Logger logger = LoggerFactory.getLogger(MinIOService.class);
    @Autowired
    private MinioClient minioClient;
    @Autowired
    private MinioPojo minioPojo;


    /**
     *
     * @param file 本地文件路径
     * @param bucketfilename minIO上文件的全路径名，如：2021/01/01/评审单.PDF
     * @return
     */
    public String uploadFile(MultipartFile file, String bucketfilename) {
        logger.info("service-uploadFile:开始上传文件到MinIO");
        String returnInfo = null;
        // 检查文件是否为空
        if (file.getSize() == 0 || file.isEmpty()) {
            throw new IllegalArgumentException("上传的文件无效或为空！");
        }

        String bucketName = minioPojo.getBucketName();
        try {
            logger.info("service-uploadFile:开始上传文件到MinIO=> " + minioPojo.getUrl() + "/" + bucketName + "/" + bucketfilename);
            logger.info("01");
            // 判断桶是否存在
            boolean bucketExists = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
            if (!bucketExists) {
                logger.info("服务-uploadFile:桶不存在，正在创建桶 => " + bucketName);
                minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
            }
            logger.info("02");
            // 使用 MultipartFile 的输入流上传文件
            try (InputStream inputStream = file.getInputStream()) {
                minioClient.putObject(PutObjectArgs.builder()
                        .bucket(bucketName)
                        .object(bucketfilename)
                        .stream(inputStream, file.getSize(), -1)
                        .contentType(file.getContentType())
                        .build());
            }
            logger.info("03");
            // 构造返回的文件访问 URL
            URI fileUri = new URI(minioPojo.getUrl() + "/" + bucketName + bucketfilename);
            returnInfo = fileUri.toString();

        }catch (IllegalArgumentException e1){
            logger.error("服务-uploadFile:参数错误", e1);
            throw new IllegalArgumentException("文件上传失败！", e1);
        }catch (Exception e) {
            logger.error("服务-uploadFile:未知错误", e);
            throw new RuntimeException("文件上传失败！", e);
        }
        return returnInfo;
    }


    /**
     * 检查给定的路径是否为一个有效的文件
     * @param filePath 文件路径
     * @return 如果路径存在且是一个文件，则返回true；否则返回false
     */
    private boolean isFileExists(String filePath) {
        File file = new File(filePath);
        return file.exists() && file.isFile();
    }

    /**
     * 根据文件名获取内容类型
     * @param file 文件对象
     * @return 内容类型
     */
    private String getContentType(File file) {
        String fileName = file.getName();
        if (fileName.endsWith(".pdf")) {
            return "application/pdf";
        } else if (fileName.endsWith(".jpg") || fileName.endsWith(".jpeg")) {
            return "image/jpeg";
        } else if (fileName.endsWith(".png")) {
            return "image/png";
        } else if (fileName.endsWith(".txt")) {
            return "text/plain";
        } else {
            return "application/octet-stream";
        }
    }

    /**
     * 删除minIO指定目录下的所有文件
     * @param directory 目录路径，形如："/2023/01/01/"
     * @return 删除成功返回true，否则返回false
     */
    public boolean deleteByDir(String directory) {
        logger.info("service-deleteByDir:开始删除目录下的所有文件 => " + directory);
        boolean result = false;
        String bucketName = minioPojo.getBucketName();
        try {
            // 列出目录下的所有对象
            Iterable<Result<Item>> results = minioClient.listObjects(
                    ListObjectsArgs.builder().bucket(bucketName).prefix(directory).build());

            for (Result<Item> resultItem : results) {
                Item item = resultItem.get();
                // 删除每个对象
                minioClient.removeObject(
                        RemoveObjectArgs.builder().bucket(bucketName).object(item.objectName()).build());
            }
            result = true;
        } catch (Exception e) {
            logger.error("服务-deleteByDir: 删除文件失败", e);
        }


        return result;
    }
}
